import javax.swing.*;
import javax.swing.table.*;
import java.io.*;

import CH.psi.midas.*;

public class OdbTableModel extends javax.swing.table.AbstractTableModel implements OdbKeyUpdate
{
  OdbKey  odbKey = null;
  OdbKey  subKey[];
  int     nSubKey;
  boolean isDir;
        
  String[] columnName = {
    "Name", "Value"
  };
  
  public int getColumnCount()
  {
    return columnName.length;
  }

  public int getRowCount()
  {
    if (odbKey == null)
      return 0;
      
    return isDir ? nSubKey : odbKey.getNumValues();
  }
  
  public String getColumnName(int columnIndex) 
  {
    return columnName[columnIndex];
  }

  public boolean isCellEditable(int rowIndex, int columnIndex) {
    return columnIndex == 1;
  }
  
  public Object getValueAt(int rowIndex, int columnIndex)
  {
    if (odbKey == null)
      return null;

    switch (columnIndex) {
      case 0: 
        if (isDir) 
          return subKey[rowIndex].getName();
        if (odbKey.getNumValues() == 1)
          return odbKey.getName();
        else
          return "["+rowIndex+"]";
        
      case 1: 
        if (isDir) {
          if (subKey[rowIndex].getType() == OdbKey.TID_KEY)
            return null;
          return subKey[rowIndex].toString();
        }
        return odbKey.toString(rowIndex);  
      
      default: return null;
    }
  }
    
  public void setValueAt(Object value, int rowIndex, int columnIndex) {
    if (columnIndex != 1)
      return;

    try {
      if (isDir) {
        subKey[rowIndex].setText(value.toString());
        subKey[rowIndex].getData();
      } else {
        odbKey.setTextAt(value.toString(), rowIndex);
        odbKey.getDataAt(rowIndex);
      }
    fireTableCellUpdated(rowIndex, columnIndex);
    } catch (OdbNoAccessException e) {
      JOptionPane.showMessageDialog(null, "Currently no write access to this key\n"+
                                          "Stop run and try again", 
                                          "MIDAS Experiment Control", 
                                          JOptionPane.ERROR_MESSAGE);
    } catch (IOException e) { System.out.println(e); }
  }
    
  public void setOdbKey(OdbKey k) {
    
    int i;
    
    // unlink old key
    try {
      if (odbKey != null)
        {
        if (isDir)
          for (i=0 ; i<nSubKey ; i++)
            subKey[i].hotUnlink();  
        else
          odbKey.hotUnlink();
        }
    } catch (Exception e) {}
    
    // save new key
    this.odbKey = k;
    isDir = k.getType() == OdbKey.TID_KEY;

    try {
      if (isDir) {
        
        subKey = new OdbKey[k.getNumValues()];
      
        // get subkeys for this key
        nSubKey = 0;
        for (i=0 ; ; i++) {
          OdbKey sk = k.enumSubKey(i);
          if (sk == null)
            break;
          if (sk.getType() != OdbKey.TID_KEY) {
            subKey[nSubKey++] = sk;  
            sk.hotLink(this);
          }
        }
      } else {
          odbKey.hotLink(this);
          subKey = null;
      }
          
    } catch (Exception e) { System.out.println(e); }
      
    fireTableDataChanged();
  }
  
  public void update(OdbKey k) {
    fireTableDataChanged();
  }
}