// -*- C++ -*-
//
// user_ana.cc
//
// Author:  Ken Sakashita
// Created: Fri Nov 29 14:34:54 JST 2002
//
// $Id: user_ana.cc,v 1.4 2002/12/09 23:58:47 kensh Exp $
//
// $Log: user_ana.cc,v $
// Revision 1.4  2002/12/09 23:58:47  kensh
// Add monitor plots for CsI clusters.
//
// Revision 1.3  2002/12/06 23:19:41  kensh
// Add histograms for # of hits.
//
// Revision 1.2  2002/12/03 18:07:07  kensh
// Add some histograms monitoring # of cluster hit.
//
// Revision 1.1  2002/11/29 05:49:07  kensh
//
// Initial version
//
//
//
//
//

#include <iostream>
#include <iomanip>
#include <list>
#include <string>
#include <unistd.h>
#include "E391Detector/E391Detector.h"
#include "ccfortran/ccfortran.h"
#include "TupleManager/TupleManager.h"
#include "detectorMonitor/user_inc.h"

//
extern TupleManager otuple;
extern E391Detector&  e391det;
extern unsigned int  TriggerBit;

// Higz
#define NWPAWC  1000000
int  pawc_[NWPAWC];


#define ADCXBIN 100
#define ADCXMIN 0.
#define ADCXMAX 8*4095.
#define TDCXBIN 100
#define TDCXMIN 0.
#define TDCXMAX 4095.

//
void
setHbook()
{
  HDELET( 0 );
  

  for( int d = 0; d < N_DETECTOR; d++ ) {
    for( int i = 0; i < N_TRIG; i++ ) {
      // ADC Hitpattern
      int Hid = Hinfo[d].BaseHid + 101 + i;
      char str[256];
      sprintf(str, "%s ADC Hitpattern : Trig=%s", Hinfo[d].Detname.c_str(), Trginfo[i].Name.c_str());
      HBOOK1( Hid, str, Hinfo[d].Xbin, Hinfo[d].Xmin, Hinfo[d].Xmax);
      // TDC Hitpattern
      Hid = Hinfo[d].BaseHid + 201 + i;
      sprintf(str, "%s TDC Hitpattern : Trig=%s", Hinfo[d].Detname.c_str(), Trginfo[i].Name.c_str());
      HBOOK1( Hid, str, Hinfo[d].Xbin, Hinfo[d].Xmin, Hinfo[d].Xmax);
      // # of hit
      Hid = Hinfo[d].BaseHid + 301 + i;
      sprintf(str, "%s # of hit : Trig=%s", Hinfo[d].Detname.c_str(), Trginfo[i].Name.c_str());
      HBOOK1( Hid, str, Hinfo[d].Xbin, Hinfo[d].Xmin, Hinfo[d].Xmax);
      
    }
  }
  // Ncluster
  for( int i = 0; i < N_TRIG; i++ ) {
    // # of cluster by TDC
    int Hid = 100 + i;
    char str[256];
    sprintf(str, "# of cluster : Trig=%s",Trginfo[i].Name.c_str());
    HBOOK1( Hid, str, 577, 0., 577.);
    // # of cluster by MTDC
    Hid = 110 + i;
    sprintf(str, "# of cluster(MTdc) : Trig=%s",Trginfo[i].Name.c_str());
    HBOOK1( Hid, str, 577, 0., 577.);    
  }
  // CsI Sum
  for( int i = 0; i < N_MAX_CSI_SUM; i++ ){
    //
    int Hid = 200 + i;
    char str[256];
    sprintf(str, "pedestal CSI SUM %02d",i);
    HBOOK1( Hid, str, 5000, 0., 5000.);
  }


}

// init routine
void
user_init()
{
  // init HIGZ
  //HLIMIT( NWPAWC );
  //HLIMAP( NWPAWC, "TEST" );
  otuple.initMap( NWPAWC, "DETM" );
  //
  setHbook();

  
}


// begin_of_run routine
void
user_begin_of_run( int run_number )
{
  //
  setHbook();
}


// process_event routine
void
user_process_event( int run_number, int spill_number,
                    int event_number, int trigger_bit )
{

}


// process_event routine
void
user_process_event_exp( HNDLE hbuf,HNDLE request_id,
                        EVENT_HEADER* pheader, void* pevent )
{
  /* ODB structures */
  extern RUNINFO  runinfo;
  
  static int  old_spillnum = 0;
  int  spillnum = pheader->serial_number;

  //
  static int  RunNumber = 0;
  if ( RunNumber != runinfo.run_number || spillnum < old_spillnum ){
    RunNumber = runinfo.run_number;
    e391det.setRunNumber( RunNumber );
    // get DB for this run
    e391det.getDB();
    //e391det.getDB( 1 );
    setHbook();
  }
  
  //
  e391det.setEvent( pheader, pevent );

  // Data event
  DWORD*  pdata;
  int  nb = e391det.bank( "NGF0",&pdata );
  if ( nb > 0 ){
    std::cout << "Run"      << runinfo.run_number
              << " spill="  << spillnum
              << " nevent=" << pdata[0]
              << std::endl;
    int  nevent = pdata[0];
    DWORD* pev;
    int nn = e391det.bank("TRG1",&pev);
    if( nn == 0 ) 
      return;

    int nOffSpill = 0;
    for( int i=0; i < nn; i++ ){
      if( pev[i] & TRIG_SPILLON ) 
	break;
      nOffSpill++;
    }
    int  nOnSpill = nevent - nOffSpill;
    int  nskip = (int)(nOnSpill / 4) + 1;
    std::list<int> evList;
    evList.push_back( (int)(nOffSpill/2) );
    
    int iev = nOffSpill + 1;
    while( 1 ) {
      evList.push_back( iev );
      if( (iev += nskip) > nevent )
	break;
    }
    
    std::cout << " nevent = " << nevent
              << " nsample = " << evList.size() << std::endl;
    static int ct  = 0;
    bool pedmon = true;

    for( std::list<int>::iterator m = evList.begin(); m != evList.end(); m++ ){
      if ( e391det.unpack( *m ) == 0 ) {
	ct += (*m);
	int trig = e391det.triggerBit( *m )&0xffff;
	std::cout << " iev = " << *m  << " trig = 0x" 
		  << std::hex << trig << std::dec << std::endl; 

	// CsI sum for trigger
	if( trig&TRIG_CLOCK ) {
	  if( pedmon ) {
	    for( int i=0; i<N_MAX_CSI_SUM; i++ ){ 
	      int rawAdc = e391det.CsI_Sum(i).adc()&0xffff;
	      int range  = e391det.CsI_Sum(i).adc()&0xffff0000;
	      int Hid    = 200 + i;
	      if( range == 0 ) 
		HFILL( Hid, 0.5 + (float)spillnum, 0., rawAdc );
	      
	    }
	    pedmon = false;
	  }
	}

	// Monitor Histograms
	for( int i=0; i<N_TRIG; i++ ) {
	  if( trig&Trginfo[i].Bit ) {
	    // N-cluster
	    unsigned long long nclus[3];
	    nclus[0] = e391det.tdcCrate(1).tdc(3).hitPattern() & 0xffff000000000000LL;
	    nclus[1] = e391det.tdcCrate(1).tdc(5).hitPattern() & 0xffffffff00000000LL;
	    nclus[2] = e391det.tdcCrate(1).tdc(7).hitPattern() & 0xffffffff00000000LL;
	    //std::cout << "nclus0=0x" << std::hex << nclus[0] << std::endl
	    //          << "nclus1=0x" << nclus[1] << std::endl
	    //          << "nclus2=0x" << nclus[2] << std::endl;
	    nclus[0] >>= 48;
	    nclus[1] >>= 32;
	    nclus[2] >>= 32;
	    int  ncluster = 0;
	    for( int ic=0;ic<3;ic++ ){
	      while( nclus[ic] != 0 ){
		ncluster += nclus[ic] & 0x1;
		nclus[ic] >>= 1;
	      }
	    }
	    // N-cluster by MultihitTDC
	    unsigned long long nmclus[2];
	    nmclus[0] = e391det.mtdcCrate(1).tdc(6).hitPatternL();
	    nmclus[1] = e391det.mtdcCrate(1).tdc(6).hitPatternH() & 0x00ffffffLL;
	    int  nmcluster = 0;
	    for( int ic=0;ic<2;ic++ ){
	      while( nmclus[ic] != 0 ){
		nmcluster += nmclus[ic] & 0x1;
		nmclus[ic] >>= 1;
	      }
	    }
	    //std::cout << "Ncluster=" << ncluster << std::endl;
	    HFILL( 100+i, (float)ncluster, 0., 1.);
	    HFILL( 110+i, (float)nmcluster, 0., 1.);

	    // CsI
	    int nhitcsi = 0;
	    for( int ch=0; ch<N_MAX_CSI; ch++ ) {
	      int rawAdc   = e391det.CsI( ch ).adc()&0xffff;
	      int rangeBit = e391det.CsI( ch ).adc()&0xffff0000;
	      int hit      = ( rangeBit ? (rawAdc - Hinfo[0].AdcCutH) : (rawAdc - Hinfo[0].AdcCutL) );
	      //int Adc      = ( rangeBit ? rawAdc*8 : rawAdc );
	      //int Tdc      = e391det.CsI( ch ).tdc();
	      
	      if( hit > 0 ) HFILL( 1101+i, 0.5 + (float)ch, 0., 1. ); 
	      if( e391det.CsI( ch ).tdc()    > 0.  ) {
		HFILL( 1201+i, 0.5 + (float)ch, 0., 1. ); 
		nhitcsi++;
	      }
	    }
	    HFILL( 1301+i, nhitcsi, 0., 1. ); 

	    // CC03
	    int nhitcc03 = 0;
	    for( int ch=0; ch<N_MAX_CC03; ch++ ) {
	      int rawAdc   = e391det.CC03( ch ).adc()&0xffff;
	      int rangeBit = e391det.CC03( ch ).adc()&0xffff0000;
	      int hit      = ( rangeBit ? (rawAdc - Hinfo[1].AdcCutH) : (rawAdc - Hinfo[1].AdcCutL) );
	      
	      if( hit > 0 ) HFILL( 2101+i, 0.5 + (float)ch, 0., 1. ); 
	      if( e391det.CC03( ch ).tdc()    > 0.  ) { 
		HFILL( 2201+i, 0.5 + (float)ch, 0., 1. ); 
		nhitcc03++;
	      }
	    }
	    HFILL( 2301+i, nhitcc03, 0., 1. ); 

	    // CV
	    int nhitcv = 0;
	    for( int ch=0; ch<N_MAX_CHARGEVETO; ch++ ) {
	      int rawAdc   = e391det.ChargeVeto( ch ).adc()&0xffff;
	      int rangeBit = e391det.ChargeVeto( ch ).adc()&0xffff0000;
	      int hit      = ( rangeBit ? (rawAdc - Hinfo[2].AdcCutH) : (rawAdc - Hinfo[2].AdcCutL) );
	      
	      if( hit > 0 ) HFILL( 3101+i, 0.5 + (float)ch, 0., 1. ); 
	      if( e391det.ChargeVeto( ch ).tdc()    > 0.  ) {
		HFILL( 3201+i, 0.5 + (float)ch, 0., 1. ); 
		nhitcv++;
	      }
	    }
	    HFILL( 3301+i, nhitcv, 0., 1. ); 

	    // Sand
	    int nhitsand = 0;
	    for( int ch=0; ch<N_MAX_SANDWICH; ch++ ) {
	      int rawAdc   = e391det.Sandwich( ch ).adc()&0xffff;
	      int rangeBit = e391det.Sandwich( ch ).adc()&0xffff0000;
	      int hit      = ( rangeBit ? (rawAdc - Hinfo[3].AdcCutH) : (rawAdc - Hinfo[3].AdcCutL) );
	      
	      if( hit > 0 ) HFILL( 4101+i, 0.5 + (float)ch, 0., 1. ); 
	      if( e391det.Sandwich( ch ).tdc()    > 0.  ) {
		HFILL( 4201+i, 0.5 + (float)ch, 0., 1. ); 
		nhitsand++;
	      }
	    }
	    HFILL( 4301+i, nhitsand, 0., 1. ); 

	    // BA
	    int nhitba = 0;
	    for( int ch=0; ch<N_MAX_BEAMCATCHER; ch++ ) {
	      int rawAdc   = e391det.BeamCatcher( ch ).adc()&0xffff;
	      int rangeBit = e391det.BeamCatcher( ch ).adc()&0xffff0000;
	      int hit      = ( rangeBit ? (rawAdc - Hinfo[4].AdcCutH) : (rawAdc - Hinfo[4].AdcCutL) );
	      
	      if( hit > 0 ) HFILL( 5101+i, 0.5 + (float)ch, 0., 1. ); 
	      if( e391det.BeamCatcher( ch ).tdc()    > 0.  ) {
		HFILL( 5201+i, 0.5 + (float)ch, 0., 1. ); 
		nhitba++;
	      }
	    }
	    HFILL( 5301+i, nhitba, 0., 1. ); 

	    // C6
	    int nhitc6 = 0;
	    for( int ch=0; ch<N_MAX_C6; ch++ ) {
	      int rawAdc   = e391det.C6( ch ).adc()&0xffff;
	      int rangeBit = e391det.C6( ch ).adc()&0xffff0000;
	      int hit      = ( rangeBit ? (rawAdc - Hinfo[5].AdcCutH) : (rawAdc - Hinfo[5].AdcCutL) );
	      
	      if( hit > 0 ) HFILL( 6101+i, 0.5 + (float)ch, 0., 1. ); 
	      if( e391det.C6( ch ).tdc()    > 0.  ) { 
		HFILL( 6201+i, 0.5 + (float)ch, 0., 1. ); 
		nhitc6++;
	      }
	    }
	    HFILL( 6301+i, nhitc6, 0., 1. ); 

	    ////
	  }
	}
	//
      }
      else {
	break;
      }
    } // End of Event Loop
  } // End of Event Data
}


// end_of_run routine
void
user_end_of_run( int run_number )
{
  //
  std::cout << "user_end_of_run" << std::endl;
  HDELET( 0 );
  //init    = false;
}


// term routine
void
user_term()
{
  //
}


