// -*- C++ -*-
//
// gukine.cc
// C++ interface of GUKINE
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Wed Jul 11 15:29:57 JST 2001
//
// $Id: Gukine.cc,v 1.1.1.1 2002/08/13 12:48:08 kensh Exp $
//
// Revision History
//
// $Log: Gukine.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:08  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <stdio.h>
#include "ccfortran/ccfortran.h"
#include "ccfortran/G321Common.h"
#include "CLHEP/Vector/ThreeVector.h"
#include "gsim-general/Ptype.h"
#include "gsim-general/GsimGeneral.h"
#include "TupleManager/TupleManager.h"
#include "ntuple.h"


//////
#define  DEBUG  0


//////
int    GenMechanism[MAX_MECH];

// FFkey
float  MoveGenZ[] = { 0,0 };


//////
void
Gukine()
{
  if ( gcflag_.ISWIT[3-1] > 0 )
    std::cerr << "Gukine" << std::endl;
  if ( gcflag_.ISWIT[3-1] > 1 )
    std::cerr << "Gukine : ikine=" << gckine_.IKINE << std::endl;

  // Event
  int  Ntrk = 0;
  if ( Ituple ){
    Ituple->read( gcflag_.IDEVT );
    float  vtx[]  = { gen.XX, gen.YY, gen.ZZ };
    float  plab[] = { gen.PX*gen.PP,gen.PY*gen.PP,gen.PZ*gen.PP };
    int    pid    = gen.PN;
    if ( MoveGenZ[0] != 0 )
      vtx[2] = MoveGenZ[1];
    int  nv=0, nt=0;
    GSVERT( vtx,0,0,0,0,nv );
    GSKINE( plab,pid,nv,0,0,nt );
    Ntrk = nt;
  }
  else if ( gckine_.IKINE == 3 ){
    // Klong on the E391a beamline
    static bool  first = true;
    if ( first ){
      GsimKlongDecayMode( (int)gckine_.PKINE[0] );
      first = false;
    }
    double  z = gckine_.PKINE[1];
    int  absorber = (int)gckine_.PKINE[2];
    Hep3Vector  p,v;
    GsimGenKlongCol6( z,absorber, p,v );
    float  vtx[]  = { v.x(), v.y(), v.z() };
    float  plab[] = { p.x(), p.y(), p.z() };
    int    pid = 10;
    int  nv=0, nt=0;
    GSVERT( vtx,0,0,0,0,nv );
    GSKINE( plab,pid,nv,0,0,nt );
    Ntrk = nt;
  }
  else{
    // PID  pmin   pmax   thmin thmax phmin phmax zmin zmax Nparticle
    int  pid;
    int  Npart = (int)gckine_.PKINE[9];
    if ( gckine_.IKINE == 1 ){
      pid = (int)gckine_.PKINE[0];
      Npart  = (int)gckine_.PKINE[9];
    }
    else {
      pid   = 10; // KL
      Npart = 1;  // Multiple KL event is not availavble for now.
      static bool  first = true;
      if ( first ){
        GsimKlongDecayMode( (int)gckine_.PKINE[0] );
        first = false;
      }
    }
    double  pmin   = gckine_.PKINE[1];
    double  pmax   = gckine_.PKINE[2];
    double  pthmin = gckine_.PKINE[3];
    double  pthmax = gckine_.PKINE[4];
    double  pphmin = gckine_.PKINE[5];
    double  pphmax = gckine_.PKINE[6];
    double  zmin   = gckine_.PKINE[7];
    double  zmax   = gckine_.PKINE[8];
    for( int i=0;i<Npart;i++ ){
      Hep3Vector  p,v;
      GsimParticleGun( pid, 
                       pmin,pmax,pthmin,pthmax,pphmin,pphmax, // plab
                       0,2, 0,360, zmin,zmax,                 // vtx
                       p,v );
      float  vtx[]  = { v.x(), v.y(), v.z() };
      float  plab[] = { p.x(), p.y(), p.z() };
      int  nv=0, nt=0;
      GSVERT( vtx,0,0,0,0,nv );
      GSKINE( plab,pid,nv,0,0,nt );
      Ntrk = nt;
    }
  }
  // show
  if ( Display && gcflag_.ISWIT[7-1]>0 && Ntrk>0 ){
    for( int i=1;i<=Ntrk;i++ ){
      float   kvert[3],pvert[4],vvert[3];
      int     pid,nvert,nubuf;
      float*  ubuf;
      int     ntbeam,nttarg;
      float   tofg;
      GFKINE( i,kvert,pvert,pid,nvert,ubuf,nubuf );
      GFVERT( nvert,vvert,ntbeam,nttarg,tofg,ubuf,nubuf );
      double  e = sqrt( pvert[0]*pvert[0]+pvert[1]*pvert[1]+pvert[2]*pvert[2]
                        + ptype[pid].mass*ptype[pid].mass );
      char  s[256];
      sprintf( s,"Event %d trk %d pid %d v(%.2f,%.2f,%.2f)"
               " p(%.4f,%.4f,%.4f) e %.4f",
               gcflag_.IDEVT, i, pid, vvert[0], vvert[1], vvert[2],
               pvert[0],pvert[1],pvert[2], e );
      IGSET( "CHHE",0.3 );
      ITX( 1,19,s );
    }
  }

  if ( gcflag_.ISWIT[3-1] > 1 )
    std::cerr << "Ntrk=" << Ntrk << std::endl;

  if ( gcflag_.ISWIT[3-1] > 0 ){
    GPRINT( "VERT", 0 );
    GPRINT( "KINE", 0 );
  }

  // Clear Data Buffer
  nresult = nresult_clear;

}

