// -*- C++ -*-
//
// GsimGenParticle.cc
// Particle information in GEANT data structure
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Fri Jul 13 18:39:42 JST 2001
//
// $Id: GsimGenParticle.cc,v 1.1.1.1 2002/08/13 12:48:08 kensh Exp $
//
// Revision History
//
// $Log: GsimGenParticle.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:08  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <string>
#include <list>
#include <math.h>
#include "ccfortran/ccfortran.h"
#include "ccfortran/G321Common.h"
#include "gsim-general/Ptype.h"
#include "gsim-general/GsimGeneral.h"
#include "gsim-general/Ntuple.h"


//////
#define  DEBUG  0


//
typedef struct{
  float  XX,YY,ZZ,PX,PY,PZ,PP,EK,MASS;
  int    PN,MO,DA1,DA2,MECH,STATUS,ID;
  std::list<int>  DA;
} GENPART;

typedef struct{
  int    ID,MO,DA1,DA2;
} GENINDEX;


//////
int
GsimGenParticle()
{
  if ( DEBUG )
    std::cerr << "GsimGenParticle" << std::endl;
  //
  
  extern int  GenMechanism[];
  float       kvert[3],pvert[4],vvert[3];
  int         pid,nvert,nubuf;
  float*      ubuf=0;
  int         ntbeam,nttarg;
  float       tofg;
  
#if 1
  static int  maxNtrack = 1000;
  static GENPART*  glist = new GENPART[maxNtrack];

  if ( maxNtrack < gcnum_.NTRACK ){
    delete [] glist;
    maxNtrack = gcnum_.NTRACK * 2;
    glist = new GENPART[maxNtrack];
  }
#else
  GENPART  glist[gcnum_.NTRACK];
#endif
  
  int  ngen = 0;
  for( int i=1;i<=gcnum_.NTRACK;i++ ){
    int  mec = ( i < MAX_MECH ) ? GenMechanism[i] : -1;
    std::string  smec( (char*)&mec,4 );
    //if ( mec == 0 || smec == "DCAY" ){
    GFKINE( i,kvert,pvert,pid,nvert,ubuf,nubuf );
    GFVERT( nvert,vvert,ntbeam,nttarg,tofg,ubuf,nubuf );
    // nvert  == itself
    // ntbeam == mother
    bool  save = ( ntbeam == 0 ) ? true : false;
    int  mo = 0;

    if ( mec == 0 || smec == "DCAY" ){
      for( int m=0; !save && m<ngen; m++ ){
        if ( glist[m].ID == ntbeam ){
          mo = m+1;
          save = true;
        }
      }
    }
    if ( save ){
      double p2 = pvert[0]*pvert[0]+pvert[1]*pvert[1]+pvert[2]*pvert[2];
      double p  = sqrt( p2 );
      double e  = sqrt( ptype[pid].mass*ptype[pid].mass + p2 );
      double ek = e - ptype[pid].mass; // Ek = m(gamma - 1) = E - m
      //std::cout << "pid" << pid << " ek" << ek << " e" << e << std::endl;
      glist[ngen].XX   = kvert[0];
      glist[ngen].YY   = kvert[1];
      glist[ngen].ZZ   = kvert[2];
      glist[ngen].PX   = pvert[0]/p;
      glist[ngen].PY   = pvert[1]/p;
      glist[ngen].PZ   = pvert[2]/p;
      glist[ngen].PP   = p;
      glist[ngen].EK   = ek;
      glist[ngen].MASS = ptype[pid].mass;
      glist[ngen].PN   = pid;
      glist[ngen].MO   = mo;
      glist[ngen].DA1  = 0;
      glist[ngen].DA2  = 0;
      glist[ngen].MECH = GenMechanism[i];
      glist[ngen].STATUS = 0;
      glist[ngen].ID   = nvert;
      glist[ngen].DA.clear();
      if ( mo > 0 ){
        glist[mo-1].DA.push_back( ngen+1 );
      }
      ngen++;
      //
      if ( DEBUG ){
        GENPART&  g = glist[ngen-1];
        std::cerr << i 
                  << " this="<< nvert
                  << " mo="  << ntbeam
                  << " ("    << mo << ")"
                  << " pid=" << g.PN
                  << " mec=" << std::string((char*)&g.MECH,4)
                  << " v("   << g.XX << "," << g.YY << "," << g.ZZ << ")"
                  << " p("   << g.PX*g.PP << "," << g.PY*g.PP 
                  << "," << g.PZ*g.PP << ")"
                  << std::endl;
      }
    }
  }
  
  // primary particle
#if 1
  static int  maxIndex = 100;
  static GENINDEX*  gindex = new GENINDEX[maxIndex];
  if ( maxIndex < ngen ){
    delete [] gindex;
    maxIndex = ngen * 2;
    gindex = new GENINDEX[maxIndex];
  }    
#else
  GENINDEX   gindex[ngen];
#endif

  int  ct = 0;
  for( int i=0;i<ngen;i++ ){
    if ( glist[i].MO == 0 ){
      gindex[ct].ID = i+1;
      gindex[ct].MO = glist[i].MO;
      //std::cerr << ct 
      //          << " new=" << gindex[ct].ID
      //          << " mo="  << gindex[ct].MO 
      //          << std::endl;
      ct++;
    }
  }
  
  // secondary particle
  int  da = ct;
  for( int i=0;i<ngen;i++ ){
    GENPART&  gm = glist[ gindex[i].ID-1 ];
    if ( gm.DA.size() > 0 ){
      gindex[i].DA1 = da + 1;
      da += gm.DA.size();
      gindex[i].DA2 = da;
      for( std::list<int>::iterator d = gm.DA.begin();
           d != gm.DA.end(); d++ ){
        gindex[ct].ID = *d;
        gindex[ct].MO = i+1;
        ct++;
      }
    }
    else{
      gindex[i].DA1 = 0;
      gindex[i].DA2 = -1;
    }
    
    if ( DEBUG ){
      std::cerr << i+1
                << " this="<< gindex[i].ID
                << " mo="  << gindex[i].MO
                << " da1=" << gindex[i].DA1
                << " da2=" << gindex[i].DA2
                << " pid=" << gm.PN
                << " mec=" << std::string((char*)&gm.MECH,4)
                << " v("   << gm.XX << "," << gm.YY 
                << "," << gm.ZZ << ")"
                << " p("   << gm.PX*gm.PP << "," 
                << gm.PY*gm.PP << "," << gm.PZ*gm.PP << ")"
                << std::endl;
    }
  }  

  // decay mode
  enum { gm=1,ep,em,nu,mp,mm,p0,pp,pm,KL };
  int  mode = 0;
  //if ( (gckine_.IKINE==2 || gckine_.IKINE==3) && gckine_.PKINE[0]>0 ){
  //  mode = (int)gckine_.PKINE[0];
  //}
  //else if ( glist[0].PN == KL && gindex[0].DA1 > 0 ){
  if ( glist[0].PN == KL && gindex[0].DA1 > 0 ){
    int  nd1 = gindex[0].DA1-1;
    int  nd2 = gindex[0].DA2-1;
    int  d1 = glist[nd1].PN;
    int  d2 = ( nd2 > nd1   ) ? glist[nd1+1].PN : 0;
    int  d3 = ( nd2 > nd1+1 ) ? glist[nd1+2].PN : 0;
    if (      (d1==pm && d2==ep) || (d2==pm && d1==ep) )
      mode = 1;
    else if ( (d1==pp && d2==em) || (d2==pp && d1==em) )
      mode = 2;
    else if ( (d1==pm && d2==mp) || (d2==pm && d1==mp) )
      mode = 3;
    else if ( (d1==pp && d2==mm) || (d2==pp && d1==mm) )
      mode = 4;
    else if ( d1==p0 && d2==p0 && d3==p0 )
      mode = 5;
    else if ( d1==p0 && d2==p0 && d3==0 )
      mode = 7;
    else if ( ( (d1==pp && d2==pm) || (d2==pp && d1==pm) ) && d3==0 )
      mode = 8;
    else if ( d1==gm && d2==gm && d3==0 )
      mode = 9;
    else if ( d1==p0 && d2==0 ) // pi0 nu nubar -> only pi0 is saved
      mode = 10;
    else if ( d1+d2+d3 == pp+pm+p0 && d1*d2*d3 == pp*pm*p0 )
      mode = 6; // really?
    else
      mode = -1;    
  }

  // copy 
  gen_particle.NGEN = ( ngen<MAX_GEN ) ? ngen : MAX_GEN;
  for( int i=0; i<ngen && i<MAX_GEN; i++ ){
    GENPART&  g = glist[ gindex[i].ID-1 ];
    gen_particle.XX[i]     = g.XX;
    gen_particle.YY[i]     = g.YY;
    gen_particle.ZZ[i]     = g.ZZ;
    gen_particle.PX[i]     = g.PX;
    gen_particle.PY[i]     = g.PY;
    gen_particle.PZ[i]     = g.PZ;
    gen_particle.PP[i]     = g.PP;
    gen_particle.EK[i]     = g.EK;
    gen_particle.MASS[i]   = g.MASS;
    gen_particle.PN[i]     = g.PN;
    gen_particle.MO[i]     = gindex[i].MO;
    gen_particle.DA1[i]    = gindex[i].DA1;
    gen_particle.DA2[i]    = gindex[i].DA2;
    gen_particle.MECH[i]   = g.MECH;
    gen_particle.STATUS[i] = mode;
  }
  
  //
  return( mode );
}
