// -*- C++ -*-
//
// GsimMain.cc
// Event-Loop control function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Fri Apr 20 15:44:51 JST 2001
//
// $Id: GsimMain.cc,v 1.1.1.1 2002/08/13 12:48:08 kensh Exp $
//
// Revision History
//
// $Log: GsimMain.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:08  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <string>
#include <list>
#include <stdlib.h>
#include <unistd.h>
#include <GsimUtils.h>
#include "ccfortran/ccfortran.h"
#include "ccfortran/G321Common.h"
#include "TupleManager/TupleManager.h"
#include "gsim-general/GsimGeneral.h"
#include "gsim-general/Ntuple.h"


////////////////
#define  DEBUG  0


////////////////
typedef struct{
  std::string  fname;
  int          nlimit,nskip,nloop;
} PROCESS_FILE;


////////////////
// global variables.
int        Display = 1;
TMNtuple*  Ituple    = NULL;
TMNtuple*  Otuple    = NULL;
//TMNtuple*  Ogenpart  = NULL;
TMNtuple*  Ogsimrand = NULL;


////////////////
// command params; some are used as global variables.

static bool         Ohpss   = false;    // -ohpss 
int                 Ohid    = 1001;     // -ohid N
static std::string  Otitle  = "GSIM";   // -otitle TITLE
static std::string  Ofname;             // -o name
//static bool         Compress = false;  // -compress 
int                 SaveMode = 1;       // -save mode

static int          Rindex  = 1;        // -rindex N
static int          Rseed1  = 0;        // -rseeds seed1 seed2
static int          Rseed2  = 0;        // -rseeds seed1 seed2

static bool         Ihpss   = false;    // -ihpss
int                 Ihid    = 601;      // -ihid N
static int          Gen     = 0;        // -gen event skip
static int          GenSkip = 0;        // -gen event skip
static std::list<PROCESS_FILE> Ifiles; // -i name limit skip loop

static std::string  Psname;            // -psfile name

static int          Ilun    = 10;
static int          Olun    = 11;
static int          Pslun   = 19;

static TupleManager  OTmanager;
static TupleManager  ITmanager;


////////////////
void
GsimUsage()
{
  std::cout << "usage: gsim [-ohpss] [-ohid h] [-otitle Title] [-o ofile]"
            << std::endl
            << "            [-save mode(0:all 1:decayed 2:decayed+hit)]"
            << std::endl
            << "            [-psfile psfile] [-display]"
            << std::endl
            << "            [-gen Nevent Nskip] [-rindex Index] [-rseeds seed1 seed2]"
            << std::endl
            << "            [-ihpss] [-ihid h]"
            << std::endl
            << "            [ -i inputfile [Nevent Nskip Nloop] -i ... ]"
            << std::endl;
}


////////////////
int
GsimParams( int argc, const char* argv[] )
{
  // Clear Gxsim flag
  isGxsim = 0;
  // Clear Display flag
  Display = 0;

  // Parse command parameters
  for( int i=1;i<argc;i++ ){
    if ( strcmp( argv[i],"-ohpss" ) == 0 )
      Ohpss = true;
    else if ( strcmp( argv[i],"-ohid" )==0 && i+1<argc && argv[i+1][0]!='-' )
      Ohid = atoi( argv[++i] );
    else if ( strcmp( argv[i],"-o" )==0 && i+1<argc && argv[i+1][0]!='-' )
      Ofname = argv[++i];
    else if ( strcmp( argv[i],"-otitle" )==0 && i+1<argc && argv[i+1][0]!='-' )
      Otitle = argv[++i];
    //else if ( strcmp( argv[i],"-compress" ) == 0 )
    //  Compress = true;
    else if ( strcmp( argv[i],"-save" )==0 && i+1<argc && argv[i+1][0]!='-' )
      SaveMode = atoi( argv[++i] );
    else if ( strcmp( argv[i],"-psfile" )==0 && i+1<argc && argv[i+1][0]!='-'){
      Psname = argv[++i];
      Display = 1;
    }
    else if ( strcmp( argv[i],"-display" ) == 0 )
      Display = 1;
    else if ( strcmp( argv[i],"-gen" )==0 && i+1<argc && argv[i+1][0]!='-'){
      Gen = atoi( argv[++i] );
      if ( i+1<argc && argv[i+1][0] != '-' )
        GenSkip = atoi( argv[++i] );
    }
    else if ( strcmp( argv[i],"-rindex" )==0 && i+1<argc && argv[i+1][0]!='-' )
      Rindex = atoi( argv[++i] );
    else if ( strcmp(argv[i],"-rseeds" )==0 && i+2<argc && 
              argv[i+1][0]!='-' && argv[i+2][0]!='-' ){
      Rseed1 = atoi( argv[++i] );
      Rseed2 = atoi( argv[++i] );
    }
    else if ( strcmp( argv[i],"-ihpss" )==0 )
      Ihpss = true;
    else if ( strcmp( argv[i],"-ihid" )==0 && i+1<argc && argv[i+1][0]!='-' )
      Ihid = atoi( argv[++i] );
    else if ( strcmp( argv[i],"-i" ) == 0 && i+1<argc && argv[i+1][0]!='-' ){
      PROCESS_FILE  f = { argv[++i],0,0,1 };
      if ( i+1<argc && argv[i+1][0] != '-' )
        f.nlimit = atoi( argv[++i] );
      if ( i+1<argc && argv[i+1][0] != '-' )
        f.nskip = atoi( argv[++i] );
      if ( i+1<argc && argv[i+1][0] != '-' )
        f.nloop = atoi( argv[++i] );
      Ifiles.push_back( f );
    }
    else{
      GsimUsage();
      exit(0);
    }
  }
  // Dump Options
  std::cout << "GsimParams: options list :" << std::endl;
  if ( Ohpss ) 
    std::cout << " -ohpss" << std::endl;
  if ( !Ofname.empty() ){
    std::cout << " -ohid     = " << Ohid     << std::endl;
    std::cout << " -otitle   = " << Otitle   << std::endl;
    std::cout << " -ofile    = " << Ofname   << std::endl;
    std::cout << " -savemode = " << SaveMode << std::endl;
  }
  if ( !Psname.empty() )
    std::cout << " -psfile   = " << Psname   << std::endl;
  if ( Display )
    std::cout << " -display" <<std::endl;
  if ( Gen > 0 )
    std::cout << " -gen      = " << Gen << " " << GenSkip << std::endl;
  std::cout << " -rindex   = " << Rindex << std::endl;
  std::cout << " -rseeds   = " << Rseed1 << " " << Rseed2 << std::endl;
  if ( Ihpss )
    std::cout << " -ihpss" << std::endl;
  if ( Ifiles.size() > 0 ){
    std::cout << " -ihid     = " << Ihid << std::endl;
    for( std::list<PROCESS_FILE>::iterator i = Ifiles.begin();
         i != Ifiles.end(); i++ ){
      std::cout << " -i " << i->fname << " " << i->nlimit
                << " " << i->nskip << " " << i->nloop
                << std::endl;
    }
  }

  //
  return( 0 );
}


////////////////
int
GsimEventLoop()
{
  // Init Random Number
  if ( Rindex <= 0 ){
    std::cout << "GsimEventLoop : Rindex=" << Rindex
              << " : reset to 1" << std::endl;
    Rindex = 1;
  }
  int  seq = 1;
  if ( Rseed1 <= 0 && Rseed2 <= 0 ){ // seed not specified
    RANECQ( Rseed1, Rseed2, Rindex, "Q" );
    //std::cout << "seed=" <<  seed1 << "," << seed2 << std::endl;
  }
  RANECQ( Rseed1, Rseed2, seq, "S" );
  std::cout << "GsimEventLoop : RANECQ :"
            << " seed = " << Rseed1 << ", " << Rseed2 << std::endl;
  
  // Open PSfile
  if ( !Psname.empty() ){
    OPEN_PSFILE( Pslun,Psname );
  }

  // Test output file
  if ( access( Ofname.c_str(),F_OK ) == 0 ){ // File already exixts
    std::cout << "GsimEventLoop : Output file exists:[" << Ofname << "]" 
              << std::endl;
    return( -1 );
  }
  // Open output file
  if ( !Ofname.empty() ){
    std::string  hp = ( Ohpss ) ? "/hpss:" : "";
    if ( OTmanager.wopen( hp+Ofname,"OUT",Olun ) == 0 ){
      Otuple = OTmanager.ntuple( Ohid,Otitle,"NRESULT",
                                 nresult_ptr,s_nresult_ptr );
      //Ogenpart = OTmanager.ntuple( Ohid+1,Otitle,"NRESULT",
      //                             &gen_particle,s_gen_particle );
      Ogsimrand = OTmanager.ntuple( Ohid+2,Otitle,"NRESULT",
                                    &gsim_rand,s_gsim_rand );
    }
  }
  
  ////////////////
  // Start Event processing
  
  // Statstics
  int     neventtotal   = 0;
  int     nskiptotal    = 0;
  int     nprocesstotal = 0;
  double  cputime       = 0.0;

  if ( Gen ){
    // Statstics
    neventtotal = Gen;
    std::cout << "GsimEventLoop : Event processing started."
              << std::endl
              << " Nskip="    << GenSkip
              << " Ngenerate=" << Gen
              << std::endl;
    // Skip
    int  idevt = GsimEventSkip( GenSkip );
    
    // CPU time 
    double  starttime = CPUtime();
    
    // Event Loop 
    for( int n=1; n<=Gen; n++ ){
      idevt = GsimEvent( 1 );
      nprocesstotal++;
      if ( n % 100000 == 0 ){
        std::cout << "GsimEventLoop : Event" << n
                  << " (" << idevt << ")" 
                  << std::endl;
        std::cout.flush();
      }
    }

    // CPU time
    double  termtime = CPUtime();
    
    // Summary
    cputime += termtime - starttime;
    std::cout << "GsimEventLoop : Event processing terminated." << std::endl
              << "========================================" << std::endl
              << " Total events     : " << neventtotal      << std::endl
              << " Processed events : " << nprocesstotal    << std::endl
              << " CPU time (sec)   : " << cputime          << std::endl
              << "========================================" << std::endl
              << std::endl;

  }
  else{
    for( std::list<PROCESS_FILE>::iterator i = Ifiles.begin();
         i != Ifiles.end(); i++ ){
      
      // Test input file
      if ( access( i->fname.c_str(),F_OK ) != 0 ){// Input file does NOT exist.
        std::cout << "GsimEventLoop : No such file [" << i->fname << "]" 
                  << std::endl;
        return( -1 );
      }
      // Open input file
      std::string  hp = ( Ihpss ) ? "/hpss:" : "";
      ITmanager.ropen( hp+i->fname, "IN", Ilun );
      Ituple = ITmanager.ntuple( Ihid,"NRESULT",gen_ptr );
      int  nevent = Ituple->totalEntries();
      
      // CPU time 
      double  starttime = 0;
      double  termtime  = 0;
      
      // statistics
      std::cout << "GsimEventLoop : Event processing started."
                << std::endl
                << " Nevent="   << nevent 
                << " Nskip="    << i->nskip
                << " Nprocess=" << i->nlimit
                << " Nloop="    << i->nloop
                << std::endl;
        
      // Process file
      if ( nevent > 0 && nevent > i->nskip && i->nloop > 0 ){
        neventtotal += nevent;
        nskiptotal  += i->nskip;

        // Skip
        int  idevt = GsimEventSkip( i->nskip );

        // CPU time 
        starttime = CPUtime();
      
        // Event Loop 
        for( int n=0; idevt<nevent && (i->nlimit<=0 || n<i->nlimit); n++ ){
          if ( n % 100000 == 0 ){
            std::cout << "GsimEventLoop : Event" << n
                      << " (" << idevt << ")" 
                      << std::endl;
            std::cout.flush();
          }
          for( int loop=i->nloop;loop>0;loop-- ){
            idevt = GsimEvent( loop );
            nprocesstotal++;
          }
        }

        // CPU time
        termtime = CPUtime();
      }

      // Close event file
      ITmanager.close();
     
      // Summary
      cputime += termtime - starttime;
      std::cout << "GsimRun : Event processing terminated."   << std::endl
                << "========================================" << std::endl
                << " Total events     : " << neventtotal      << std::endl
                << " Skipped events   : " << nskiptotal       << std::endl
                << " Processed events : " << nprocesstotal    << std::endl
                << " CPU time (sec)   : " << cputime          << std::endl
                << "========================================" << std::endl
                << std::endl;
    }
  
  }

  // Close output file
  if ( Otuple ){
    OTmanager.close();
  }

  // Close PSfile
  if ( !Psname.empty() ){
    CLOSE_PSFILE( Pslun );
  }

  //
  return( 0 );
}


