/* -*- C -*-
  
   gufcntl.c
  
   Author:  Mitsuhiro YAMAGA
   Created: Wed Dec 11 22:57:06 JST 2002
  
   $Id: gufcntl.c,v 1.1 2002/12/11 14:12:33 yamaga Exp $
  
   Revision History
  
   $Log: gufcntl.c,v $
   Revision 1.1  2002/12/11 14:12:33  yamaga
   Imported package.

  
*/
/*                    Grand Unified FCNTL
         Switch multiple protocols by looking at the protocol field
         which start with / and end with : placed at the head of
         the file name.   Currently supported protocols are
         FS(file system) and HPSS(high performance storage system).

         /hpss:<hpss name>  -> invoke hpss calls for the file <hpss name>
         /:                 -> expanded to /.../cc.kek.jp/fs
                                              (hard wired for the momtne)
                               and use the system standard fcntl call.
         anything else      -> use the system standard fcntl call.

	 
         Author M. Aoki (2001/03/22).
*/	 

static char rcsid_gufcntl[] = "@(#) $Id: gufcntl.c,v 1.1 2002/12/11 14:12:33 yamaga Exp $";

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <stdlib.h>
#include "hpss_api.h"
#include "hpss_types.h"
#include "u_signed64.h"
#include "gufio/hpss_bufapi.h"
#include "gufio/gufio.h"


int gufcntl(const int fd, const int cmd, const int arg)
{
  struct gufio_tbl *thetbl;
  int fd2;

  if ( cmd == F_DUPFD ) {
    thetbl = gutblget(fd);
    if ( thetbl != NULL ) {
      fd2 = fcntl(fd, F_DUPFD, arg);
      if ( fd2 >= 0 ) {
	if ( thetbl->ftype == GUFILE_HPSS ) {
	  if ( gutblput(fd2,thetbl->ftype,thetbl->hfd) == NULL ) {
	    errno = EBADF;
	    return(-1);
	  }
	} else {
	  if ( gutblput(fd2,thetbl->ftype,fd2) == NULL ) {
	    errno = EBADF;
	    return(-1);
	  }
	}
      } else {
	return(-1);
      }
    } else {
      errno = EBADF;
      return(-1);
    }
  } else {
    errno = EINVAL;
    return(-1);
  }
  return(fd2);
}
