/* -*- C -*-

   gufio.c

   Author:  Mitsuhiro YAMAGA
   Created: Wed Dec 11 22:57:06 JST 2002

   $Id: gufio.c,v 1.1 2002/12/11 14:12:33 yamaga Exp $

   Revision History

   $Log: gufio.c,v $
   Revision 1.1  2002/12/11 14:12:33  yamaga
   Imported package.


*/
/*                    Grand Unified File IO (GUFIO) utility routines
         GUFIO switches multiple protocols by looking at the protocol field,
         which start with / and end with : placed at the head of
         the file name.   Currently supported protocols are
         FS(file system) and HPSS(high performance storage system).

         /hpss:<hpss name>  -> invoke hpss calls for the file <hpss name>
         /:                 -> expanded to /.../cc.kek.jp/fs
                                              (hard wired for the momtne)
                               and use the system standard open call.
         anything else      -> use the system standard open call.

*/	 

static char rcsid_gufio[] = "@(#) $Id: gufio.c,v 1.1 2002/12/11 14:12:33 yamaga Exp $";


#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include <sys/time.h>
#include "u_signed64.h"
#include "hpss_api.h"
#include "gufio/gufio.h"
#include <stdarg.h>

static struct gufio_tbl *thefirsttbl = NULL;



struct gufio_tbl *gutblput(int fd, gufio_t ftype, int hfd)
{
  struct gufio_tbl *ptr, *this_ptr;

  if ( thefirsttbl == NULL ) {
    if ( ( thefirsttbl = gutblinit() ) == NULL ) {
      return(NULL);
    }
  }

  ptr = thefirsttbl;
  do {
    this_ptr = ptr;
    if ( ptr->ftype == GUFILE_NONE ) {
      break;
    }
    if ( ptr->fd == fd ) {
      errno = EBADF; /* Bad file descriptor */
      return(NULL);
    }
  } while ( (ptr = ptr->next) != NULL );

  if ( this_ptr->ftype == GUFILE_NONE ) {
    ;
  } else if ( this_ptr->next == NULL ) {
    this_ptr->next = (struct gufio_tbl *)malloc(sizeof(struct gufio_tbl));
    if ( this_ptr->next == NULL ) {
      errno = ENOMEM; /* Not enough space */
      return(NULL);
    }
    this_ptr = this_ptr->next;
    this_ptr->next = NULL;
  } else {
    errno = EBADF; /* Bad file descriptor */
    return(NULL);
  }

  this_ptr->fd = fd;
  this_ptr->ftype = ftype;
  this_ptr->hfd = hfd;

  return(this_ptr);
}    




int gutblrm(int fd)
{
  struct gufio_tbl *ptr;

  if ( thefirsttbl == NULL ) {
    if ( ( thefirsttbl = gutblinit() ) == NULL ) {
      return(-1);
    }
  }

  ptr = thefirsttbl;
  do {
    if ( ptr->fd == fd ) {
      if ( ptr->ftype == GUFILE_NONE ) {
	errno = EBADF; /* Bad file descriptor */
	return(-1);
      }
      ptr->ftype = GUFILE_NONE;
      return(0);
    }
  } while ( (ptr = ptr->next) != NULL );

  errno = EBADF; /* Bad file descriptor */
  return(-1);

} 




struct gufio_tbl *gutblget(int fd)
{
  struct gufio_tbl *ptr;

  if ( thefirsttbl == NULL ) {
    if ( ( thefirsttbl = gutblinit() ) == NULL ) {
      return(NULL);
    }
  }

  ptr = thefirsttbl;
  do {
    if ( ptr->fd == fd ) {
      if ( ptr->ftype == GUFILE_NONE ) {
	errno = EBADF; /* Bad file descriptor */
	return(NULL);
      }
      return(ptr);
    }
  } while ( (ptr = ptr->next) != NULL );

  errno = EBADF; /* Bad file descriptor */
  return(NULL);

} 




struct gufio_tbl *gutblnext( struct gufio_tbl* ptr )
{
  if ( ptr != NULL ) {
    while ( (ptr = ptr->next) != NULL ){
      if ( ptr->ftype != GUFILE_NONE ) {
        break;
      }
    } 
  }
  return(ptr);

} 




struct gufio_tbl *gutblfirst()
{
  if ( thefirsttbl == NULL ) {
    if ( ( thefirsttbl = gutblinit() ) == NULL ) {
      return(NULL);
    }
  }

  return(thefirsttbl);
}




struct gufio_tbl *gutblinit()
{
  struct gufio_tbl *ptr;

  thefirsttbl = (struct gufio_tbl *)malloc(sizeof(struct gufio_tbl));
  if ( thefirsttbl == NULL ) {
    errno = ENOMEM; /* Not enough space */
    return(NULL);
  }
  ptr = thefirsttbl;

  /* Register stdin, stdout, stderr */
  /* stdin */
  ptr->fd = 0;
  ptr->ftype = GUFILE_FS;
  ptr->hfd = 0;
  ptr->next = (struct gufio_tbl *)malloc(sizeof(struct gufio_tbl));
  if ( ptr->next == NULL ) {
    errno = ENOMEM; /* Not enough space */
    return(NULL);
  }

  /* stdout */
  ptr = ptr->next;
  ptr->fd = 1;
  ptr->ftype = GUFILE_FS;
  ptr->hfd = 1;
  ptr->next = (struct gufio_tbl *)malloc(sizeof(struct gufio_tbl));
  if ( ptr->next == NULL ) {
    errno = ENOMEM; /* Not enough space */
    return(NULL);
  }

  /* stdout */
  ptr = ptr->next;
  ptr->fd = 2;
  ptr->ftype = GUFILE_FS;
  ptr->hfd = 2;
  ptr->next = NULL;

  return(thefirsttbl);
}

