/* -*- C -*-

   gulseek.c

   Author:  Mitsuhiro YAMAGA
   Created: Wed Dec 11 22:57:06 JST 2002

   $Id: gulseek.c,v 1.1 2002/12/11 14:12:33 yamaga Exp $

   Revision History

   $Log: gulseek.c,v $
   Revision 1.1  2002/12/11 14:12:33  yamaga
   Imported package.


*/
/*                    Grand Unified LSEEK
         Switch multiple protocols by looking at the protocol field
         which start with / and end with : placed at the head of
         the file name.   Currently supported protocols are
         FS(file system) and HPSS(high performance storage system).

         /hpss:<hpss name>  -> invoke hpss calls for the file <hpss name>
         /:                 -> expanded to /.../cc.kek.jp/fs
                                              (hard wired for the momtne)
                               and use the system standard lseek call.
         anything else      -> use the system standard lseek call.

	 
	 This code based on the hpsscat.c provided by IBM Japan (2001/01/23).
         M. Aoki modified the base code (2001/01/23).
*/	 

static char rcsid_gulseek[] = "@(#) $Id: gulseek.c,v 1.1 2002/12/11 14:12:33 yamaga Exp $";


#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include "hpss_api.h"
#include "hpss_types.h"
#include "u_signed64.h"
#include "gufio/hpss_bufapi.h"
#include "gufio/gufio.h"


off_t gulseek(int fd, off_t offset, int whence)
{
  struct gufio_tbl *thetbl;
  off_t position;

  thetbl = gutblget(fd);
  if ( thetbl->ftype == GUFILE_HPSS ) {
    position = hpss_Bufseek(thetbl->hfd, offset, whence);
    if ( position  < 0) {
      return(position); /* errno is supposed to be set by hpss_Buflseek */
    }
  } else {
    position = lseek(thetbl->hfd, offset, whence);
    if ( position  < 0) {
      return(position); /* errno is supposed to be set by lseek */
    }
  }

  return(position);
}
