// -*- C++ -*-
//
// MTdcEvent.cc
//
// Mon Jul 15 09:00:04 JST 2002
// Mitsuhiro YAMAGA
//
// $Id: MTdcEvent.cc,v 1.1 2002/12/01 14:37:26 yamaga Exp $
//
// $Log: MTdcEvent.cc,v $
// Revision 1.1  2002/12/01 14:37:26  yamaga
// Initial version for multihit TDC.
//
//
//

#include <iostream>
#include <iomanip>
#include "midas.h"
#include "unpack/MTdcModule.h"
#include "unpack/MTdcEvent.h"

#define DEBUG 0

static const EVENT_HEADER  zero_eh = {0,0,0,0,0};


//
MTdcEvent::MTdcEvent()
  : m_eventHeader( zero_eh ),
    m_serial( -1 ),
    m_crate( -1 ),
    m_flag( 0 ),
    m_valid( false )
{
  //
}


//
MTdcEvent::MTdcEvent( int crate, const EVENT_HEADER* pheader,
                      const DWORD* pdata, int banksize, int nth_event )
  : m_eventHeader( zero_eh ),
    m_serial( -1 ),
    m_crate( crate ),
    m_flag( 0 ),
    m_valid( false )
{
  //
  unpack( m_crate, pheader, pdata, banksize, nth_event );
}


//
MTdcEvent::~MTdcEvent()
{

}

//
const MTdcModule&
MTdcEvent::tdc( int pa ) const
{
  for( std::list<MTdcModule>::const_iterator i = m_tdcList.begin();
       i != m_tdcList.end(); i++ ){
    if ( i->pa() == pa )
      return( *i );
  }
  static const MTdcModule  dummy;
  return( dummy );
}

//
int
MTdcEvent::unpack( int crate, 
                   const EVENT_HEADER* pheader, const DWORD* pdata,
                   int banksize, int nth_event )
{
  // remove the previous data
  m_tdcList.clear();
  m_serial = -1;

  // crate
  m_crate = crate;

  // event header
  if ( pheader )
    m_eventHeader = *pheader;

  // start
  m_valid = false;
  for( int i=0;i<banksize;i++ ){
#if DEBUG
    std::cout << std::setw(8) << std::setfill('0') << std::hex 
              << pdata[i] << std::dec << std::endl;
#endif
    if ( pdata[i] == 0xffffffff ){
#if DEBUG
      std::cout << "Endmark found." << std::endl;
#endif
      break;
    }
    // skip
    int  serial = (pdata[i] >> 16) & 0xffff;
    int  nhit   = pdata[i] & 0xffff;
#if DEBUG
    std::cout << "Serial = " << serial << " nhit=" << nhit << std::endl;
#endif
    if ( nth_event > 0 && serial != nth_event ){
      i += nhit;
      continue;
    }
    // Nth event
    m_valid = true;
    m_serial = serial;

    // HeaderWord
    int  data = pdata[i+1];
    int  hpa     = (data >> 27) & 0x1f;
    int  hparity = (data >> 15) & 0x01;
    int  hbufnum = (data >> 11) & 0x07;
    int  hwc     = data & 0x07ff;

    // LRS1877S data structure
    MTdcModule tm( m_crate, hpa );
    tm.setHeaderWord( data );
#if DEBUG
    std::cout << " data=" << std::hex << data << std::dec 
              << " : pa=" << hpa
              << " parity=" << hparity
              << " bufnum=" << hbufnum
              << " wc="     << hwc
              << std::endl;
#endif
    
    // LRS1877S DataWord
    for( int ch=0;ch<hwc;ch++ ){
      int  data = pdata[i+1+1+ch];
#if DEBUG
      std::cout << " data=" << std::hex << data << std::dec << std::endl;
#endif
      if ( (data & 0xff000000) == 0xff000000 ){
        m_flag = data;
        break;
      }
      // LRS1877S data structure
      int  pa = (data >> 27) & 0x1f;
      if ( pa == hpa ){
        //int  parity = (data >> 26) & 0x01;
        //int  hitct  = (data >> 24) & 0x03;
        //int  ch     = (data >> 17) & 0x7f;
        //int  edge   = (data >> 16) & 0x01;
        //int  dt     =  data & 0xff;
        tm.setData( data );
        tm.hitList().push_back( data );
      }
      else{
        std::cout << "MTdcEvent::unpack : Data conflicted" 
                  << std::endl;
      }          
    }
    if ( tm.hitList().size() > 0 ){
#if DEBUG
      tm.show();
#endif
    }
    m_tdcList.push_back( tm );
    //i += nhit;
    break;
  }
#if DEBUG
  std::cout << "Event end." << std::endl;
#endif

  if ( m_valid )
    return( m_tdcList.size() ); // nth_event found.
  return( -1 ); // No such serial # in the spill.
}


//
int
MTdcEvent::nhitChannel() const
{
  int  n = 0;
  for( std::list<MTdcModule>::const_iterator m = m_tdcList.begin();
       m != m_tdcList.end(); m++ ){
    n += m->nhitChannel();
  }
  return( n );
}

//
int
MTdcEvent::nhit() const
{
  int  n = 0;
  for( std::list<MTdcModule>::const_iterator m = m_tdcList.begin();
       m != m_tdcList.end(); m++ ){
    n += m->nhit();
  }
  return( n );
}

